@@ -14,7 +14,6 @@ logger = logging.getLogger(__name__)
 
 from os import walk
 from os.path import basename
-from uuid import uuid4
 from urllib2 import Request, urlopen
 from urllib import urlencode
 from urlparse import urlsplit
@@ -27,18 +26,15 @@ from oauth2 import Consumer as OAuthConsumer, Token, Request as OAuthRequest, \
                    SignatureMethod_HMAC_SHA1
 
 from django.conf import settings
-from django.core.exceptions import MultipleObjectsReturned
 from django.contrib.auth import authenticate
 from django.contrib.auth.backends import ModelBackend
 from django.utils import simplejson
 from django.utils.importlib import import_module
-from django.db.utils import IntegrityError
 
 from social_auth.models import UserSocialAuth
+from social_auth.utils import setting
 from social_auth.store import DjangoOpenIDStore
-from social_auth.signals import pre_update, socialauth_registered, \
-                                socialauth_not_registered
-from social_auth.utils import sanitize_log_data
+from social_auth.backends.exceptions import StopPipeline
 
 
 # OpenID configuration
@@ -67,23 +63,16 @@ SESSION_NAME = 'openid'
 # key for username in user details dict used around, see get_user_details
 # method
 USERNAME = 'username'
-# get User class, could not be auth.User
-User = UserSocialAuth._meta.get_field('user').rel.to
-# username field max length
-USERNAME_MAX_LENGTH = User._meta.get_field(USERNAME).max_length
 
-# a few settings values
-def _setting(name, default=None):
-    return getattr(settings, name, default)
-
-CREATE_USERS = _setting('SOCIAL_AUTH_CREATE_USERS', True)
-ASSOCIATE_BY_MAIL = _setting('SOCIAL_AUTH_ASSOCIATE_BY_MAIL', False)
-LOAD_EXTRA_DATA = _setting('SOCIAL_AUTH_EXTRA_DATA', True)
-FORCE_RANDOM_USERNAME = _setting('SOCIAL_AUTH_FORCE_RANDOM_USERNAME', False)
-USERNAME_FIXER = _setting('SOCIAL_AUTH_USERNAME_FIXER', lambda u: u)
-DEFAULT_USERNAME = _setting('SOCIAL_AUTH_DEFAULT_USERNAME')
-CHANGE_SIGNAL_ONLY = _setting('SOCIAL_AUTH_CHANGE_SIGNAL_ONLY', False)
-UUID_LENGHT = _setting('SOCIAL_AUTH_UUID_LENGTH', 16)
+PIPELINE = setting('SOCIAL_AUTH_PIPELINE', (
+                'social_auth.backends.pipeline.social.social_auth_user',
+                'social_auth.backends.pipeline.associate.associate_by_email',
+                'social_auth.backends.pipeline.user.get_username',
+                'social_auth.backends.pipeline.user.create_user',
+                'social_auth.backends.pipeline.social.associate_user',
+                'social_auth.backends.pipeline.social.load_extra_data',
+                'social_auth.backends.pipeline.user.update_user_details',
+           ))
 
 
 class SocialAuthBackend(ModelBackend):
@@ -108,168 +97,50 @@ class SocialAuthBackend(ModelBackend):
         response = kwargs.get('response')
         details = self.get_user_details(response)
         uid = self.get_user_id(details, response)
-        is_new = False
         user = kwargs.get('user')
+        request = kwargs.get('request')
 
-        try:
-            social_user = self.get_social_auth_user(uid)
-        except UserSocialAuth.DoesNotExist:
-            if user is None:  # new user
-                if not CREATE_USERS or not kwargs.get('create_user', True):
-                    # Send signal for cases where tracking failed registering
-                    # is useful.
-                    socialauth_not_registered.send(sender=self.__class__,
-                                                   uid=uid,
-                                                   response=response,
-                                                   details=details)
-                    return None
-
-                email = details.get('email')
-                if email and ASSOCIATE_BY_MAIL:
-                    # try to associate accounts registered with the same email
-                    # address, only if it's a single object. ValueError is
-                    # raised if multiple objects are returned
-                    try:
-                        user = User.objects.get(email=email)
-                    except MultipleObjectsReturned:
-                        raise ValueError('Not unique email address supplied')
-                    except User.DoesNotExist:
-                        user = None
-                if not user:
-                    username = self.username(details)
-                    logger.debug('Creating new user with username %s and email %s',
-                                 username, sanitize_log_data(email))
-                    user = User.objects.create_user(username=username,
-                                                    email=email)
-                    is_new = True
-
+        # Pipeline:
+        #   Arguments:
+        #       request, backend, social_user, uid, response, details
+        #       user, is_new, args, kwargs
+        kwargs = kwargs.copy()
+        kwargs.update({
+            'backend': self,
+            'request': request,
+            'uid': uid,
+            'user': user,
+            'social_user': None,
+            'response': response,
+            'details': details,
+            'is_new': False,
+        })
+        for name in PIPELINE:
+            mod_name, func_name = name.rsplit('.', 1)
             try:
-                social_user = self.associate_auth(user, uid, response, details)
-            except IntegrityError:
-                # Protect for possible race condition, those bastard with FTL
-                # clicking capabilities
-                social_user = self.get_social_auth_user(uid)
-
-        # Raise ValueError if this account was registered by another user.
-        if user and user != social_user.user:
-            raise ValueError('Account already in use.', social_user)
-        user = social_user.user
-
-        # Flag user "new" status
-        setattr(user, 'is_new', is_new)
-
-        # Update extra_data storage, unless disabled by setting
-        if LOAD_EXTRA_DATA:
-            extra_data = self.extra_data(user, uid, response, details)
-            if extra_data and social_user.extra_data != extra_data:
-                social_user.extra_data = extra_data
-                social_user.save()
-
-        user.social_user = social_user
-
-        # Update user account data.
-        self.update_user_details(user, response, details, is_new)
-        return user
-
-    def username(self, details):
-        """Return an unique username, if SOCIAL_AUTH_FORCE_RANDOM_USERNAME
-        setting is True, then username will be a random USERNAME_MAX_LENGTH
-        chars uuid generated hash
-        """
-        def mk_uuid():
-            """Return hash from unique string"""
-            return uuid4().get_hex()
-
-        if FORCE_RANDOM_USERNAME:
-            username = mk_uuid()
-        elif details.get(USERNAME):
-            username = details[USERNAME]
-        elif DEFAULT_USERNAME:
-            username = DEFAULT_USERNAME
-            if callable(username):
-                username = username()
-        else:
-            username = mk_uuid()
-
-        short_username = username[:USERNAME_MAX_LENGTH - UUID_LENGHT]
-        final_username = None
-
-        while True:
-            final_username = USERNAME_FIXER(username)[:USERNAME_MAX_LENGTH]
-
-            try:
-                User.objects.get(username=final_username)
-            except User.DoesNotExist:
-                break
+                mod = import_module(mod_name)
+            except ImportError:
+                logger.exception('Error importing pipeline %s', name)
             else:
-                # User with same username already exists, generate a unique
-                # username for current user using username as base but adding
-                # a unique hash at the end. Original username is cut to avoid
-                # the field max_length.
-                username = short_username + mk_uuid()[:UUID_LENGHT]
+                pipeline = getattr(mod, func_name, None)
+                if callable(pipeline):
+                    try:
+                        kwargs.update(pipeline(*args, **kwargs) or {})
+                    except StopPipeline:
+                        break
 
-        return final_username
-
-    def associate_auth(self, user, uid, response, details):
-        """Associate a Social Auth with an user account."""
-        return UserSocialAuth.objects.create(user=user, uid=uid,
-                                             provider=self.name)
+        social_user = kwargs.get('social_user')
+        if social_user:
+            # define user.social_user attribute to track current social
+            # account
+            user = social_user.user
+            user.social_user = social_user
+            return user
 
     def extra_data(self, user, uid, response, details):
         """Return default blank user extra data"""
         return ''
 
-    def update_user_details(self, user, response, details, is_new=False):
-        """Update user details with (maybe) new data. Username is not
-        changed if associating a new credential."""
-        changed = False  # flag to track changes
-
-        # check if values update should be left to signals handlers only
-        if not CHANGE_SIGNAL_ONLY:
-            logger.debug('Updating user details for user %s', user,
-                         extra=dict(data=details))
-
-            for name, value in details.iteritems():
-                # do not update username, it was already generated by
-                # self.username(...) and loaded in given instance
-                if name != USERNAME and value and value != getattr(user, name,
-                                                                   None):
-                    setattr(user, name, value)
-                    changed = True
-
-        # Fire a pre-update signal sending current backend instance,
-        # user instance (created or retrieved from database), service
-        # response and processed details.
-        #
-        # Also fire socialauth_registered signal for newly registered
-        # users.
-        #
-        # Signal handlers must return True or False to signal instance
-        # changes. Send method returns a list of tuples with receiver
-        # and it's response.
-        signal_response = lambda (receiver, response): response
-
-        kwargs = {'sender': self.__class__, 'user': user,
-                  'response': response, 'details': details}
-        changed |= any(filter(signal_response, pre_update.send(**kwargs)))
-
-        # Fire socialauth_registered signal on new user registration
-        if is_new:
-            changed |= any(filter(signal_response,
-                                  socialauth_registered.send(**kwargs)))
-
-        if changed:
-            user.save()
-
-    def get_social_auth_user(self, uid):
-        """Return social auth user instance for given uid for current
-        backend.
-
-        Riase DoesNotExist exception if no entry.
-        """
-        return UserSocialAuth.objects.select_related('user')\
-                                     .get(provider=self.name, uid=uid)
-
     def get_user_id(self, details, response):
         """Must return a unique ID from values returned on details"""
         raise NotImplementedError('Implement in subclass')
@@ -284,13 +155,6 @@ class SocialAuthBackend(ModelBackend):
         """
         raise NotImplementedError('Implement in subclass')
 
-    def get_user(self, user_id):
-        """Return user instance for @user_id"""
-        try:
-            return User.objects.get(pk=user_id)
-        except User.DoesNotExist:
-            return None
-
 
 class OAuthBackend(SocialAuthBackend):
     """OAuth authentication backend base class.
@@ -316,7 +180,7 @@ class OAuthBackend(SocialAuthBackend):
         extra_data field"""
         data = {'access_token': response.get('access_token', '')}
         name = self.name.replace('-', '_').upper()
-        names = (self.EXTRA_DATA or []) + _setting(name + '_EXTRA_DATA', [])
+        names = (self.EXTRA_DATA or []) + setting(name + '_EXTRA_DATA', [])
         data.update((alias, response.get(name)) for name, alias in names)
         return data
 
@@ -325,15 +189,6 @@ class OpenIDBackend(SocialAuthBackend):
     """Generic OpenID authentication backend"""
     name = 'openid'
 
-    def get_social_auth_user(self, uid):
-        """Return social auth user instance for given uid. OpenId uses
-        identity_url to identify the user in a unique way and that value
-        identifies the provider too.
-
-        Riase DoesNotExist exception if no entry.
-        """
-        return UserSocialAuth.objects.select_related('user').get(uid=uid)
-
     def get_user_id(self, details, response):
         """Return user unique id provided by service"""
         return response.identity_url
@@ -404,8 +259,8 @@ class OpenIDBackend(SocialAuthBackend):
         SREG_ATTR, OLD_AX_ATTRS or AX_SCHEMA_ATTRS
         """
         name = self.name.replace('-', '_').upper()
-        sreg_names = _setting(name + '_SREG_EXTRA_DATA')
-        ax_names = _setting(name + '_AX_EXTRA_DATA')
+        sreg_names = setting(name + '_SREG_EXTRA_DATA')
+        ax_names = setting(name + '_AX_EXTRA_DATA')
         data = self.values_from_response(response, sreg_names, ax_names)
         return data
 
@@ -486,8 +341,8 @@ class OpenIdAuth(BaseAuth):
 
     def trust_root(self):
         """Return trust-root option"""
-        return _setting('OPENID_TRUST_ROOT',
-                        self.request.build_absolute_uri('/'))
+        return setting('OPENID_TRUST_ROOT') or \
+               self.request.build_absolute_uri('/')
 
     def auth_complete(self, *args, **kwargs):
         """Complete auth process"""
@@ -652,8 +507,8 @@ class ConsumerBasedOAuth(BaseOAuth):
         """Return tuple with Consumer Key and Consumer Secret for current
         service provider. Must return (key, secret), order *must* be respected.
         """
-        return _setting(self.SETTINGS_KEY_NAME), \
-               _setting(self.SETTINGS_SECRET_NAME)
+        return setting(self.SETTINGS_KEY_NAME), \
+               setting(self.SETTINGS_SECRET_NAME)
 
     @classmethod
     def enabled(cls):
@@ -728,19 +583,19 @@ class BaseOAuth2(BaseOAuth):
         """Return tuple with Consumer Key and Consumer Secret for current
         service provider. Must return (key, secret), order *must* be respected.
         """
-        return _setting(self.SETTINGS_KEY_NAME), \
-               _setting(self.SETTINGS_SECRET_NAME)
+        return setting(self.SETTINGS_KEY_NAME), \
+               setting(self.SETTINGS_SECRET_NAME)
 
 
 # import sources from where check for auth backends
 SOCIAL_AUTH_IMPORT_SOURCES = (
     'social_auth.backends',
     'social_auth.backends.contrib',
-) + _setting('SOCIAL_AUTH_IMPORT_BACKENDS', ())
+) + setting('SOCIAL_AUTH_IMPORT_BACKENDS', ())
 
 def get_backends():
     backends = {}
-    enabled_backends = _setting('SOCIAL_AUTH_ENABLED_BACKENDS')
+    enabled_backends = setting('SOCIAL_AUTH_ENABLED_BACKENDS')
 
     for mod_name in SOCIAL_AUTH_IMPORT_SOURCES:
         try:
